import sqlite3
from COMPS.Data import Simulation, QueryCriteria
from COMPS import Client
import os

"""
traverse all of latest_experiment folder...
load all sim_ids and get tag info from COMPS...
read in final_prev...
populate new db with sim_id, tag ids, final_prev
"""
db = "results.db"
x_tag = "slow_seek"
y_tag = "fast_seek"

def get_data():
    Client.login( "https://comps.idmod.org" )
    groupby_tags = [ x_tag, y_tag, "Run_Number" ]
    all_results = []
    exp_path = os.path.join( "latest_experiment", "ref" )
    for sim_id in os.listdir( exp_path ):
        poi = { "sim_id": sim_id }
        sim = Simulation.get(sim_id, QueryCriteria().select_children(['tags']))
        for tag in groupby_tags:
            value = sim.tags[tag]
            poi[ tag ] = float(value)
        with open( os.path.join( exp_path, sim_id, "final_prev" ) ) as fp:
            output = float( fp.read().strip() )
            poi[ "output" ] = output
        #print( f"tag, value = {tag}, {value}." )
        print( poi )
        all_results.append( poi )

    con = sqlite3.connect( db )
    cur = con.cursor()
    cur.execute( f"CREATE TABLE results ( SIM_ID TEXT, Run_Number INT, {x_tag} DECIMAL, {y_tag} DECIMAL, output DECIMAL )" )
    for result in all_results:
        insert_str = f"INSERT INTO results (SIM_ID, Run_Number, {x_tag}, {y_tag}, output ) VALUES( '{result['sim_id']}', {int(result['Run_Number'])}, {result['{x_tag}']}, {result['{y_tag}']}, {result['output']} )"
        #print( insert_str )
        cur.execute( insert_str )
    con.commit()

def plot_from_sql():
    import matplotlib.pyplot as plt
    from matplotlib import cm
    from mpl_toolkits.mplot3d import Axes3D

    import numpy as np
    fig = plt.figure()

    ax = Axes3D(fig)

    #ax = fig.gca(projection='3d')

    query = f"select {x_tag}, {y_tag}, avg(output) from results where cast({x_tag} as decimal) > 0.01 group by {x_tag}, {y_tag};"
    con = sqlite3.connect( db )
    cur = con.cursor()
    cur.execute( query )
    results = cur.fetchall()
    x = []
    y = []
    z = []
    for result in results:
        x.append( result[0] )
        y.append( result[1] )
        z.append( result[2] )
    surf = ax.plot_trisurf(x, y, z, cmap=cm.jet, linewidth=0.1)
    ax.set_xlabel( f"{x_tag} rate" )
    ax.set_ylabel( f"{y_tag} rate" )
    ax.set_zlabel( "final prevalence" )
    fig.colorbar(surf, shrink=0.5, aspect=5)
    #X, Y = np.meshgrid(np.array(x), np.array(y)) 
    #ax.plot_surface( X, Y, np.array(z) )
    plt.show()

def plot_2dcolormesh():
    """
    Work in progress, not working yet
    """
    import matplotlib.pyplot as plt
    from matplotlib import cm
    from mpl_toolkits.mplot3d import Axes3D
    from matplotlib.colors import BoundaryNorm
    from matplotlib.ticker import MaxNLocator
    import numpy as np

    fig,ax = plt.subplots()

    query = f"select {x_tag}, {y_tag}, avg(output) from results where cast({x_tag} as decimal) > 0.01 group by {x_tag}, {y_tag};"
    con = sqlite3.connect( db )
    cur = con.cursor()
    cur.execute( query )
    results = cur.fetchall()
    x = []
    y = []
    z = []
    x_set = set()
    y_set = set()
    for result in results:
        x.append( result[0] )
        x_set.add( result[0] )
        y.append( result[1] )
        y_set.add( result[1] )
        z.append( result[2] )
    twodz = []
    counter = 0
    try:
        for _x in x_set:
            row = []
            for _y in y_set:
                row.append( z[ counter ] )
                counter += 1
            twodz.append( row )
    except Exception as ex:
        print( str( ex ) )
        import pdb
        pdb.set_trace()
    cmap = plt.get_cmap('PiYG')
    levels = MaxNLocator(nbins=15).tick_values(min(z), max(z))
    norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)
    im = ax.pcolormesh(x_set, y_set, twodz, cmap=cmap, norm=norm)
    fig.colorbar(im, ax=ax)
    ax.set_xlabel( f"{x_tag} rate" )
    ax.set_ylabel( f"{y_tag} rate" )
    ax.set_zlabel( "final prevalence" )
    plt.show()

if __name__ == "__main__":
    get_data()
    plot_from_sql()

